using System;
using System.Collections;
using System.ComponentModel;
using System.Text;

namespace NinjaTrader.Data
{
	/// <summary>
    /// Bar Type that every open and close will occur on a psychological "Round Number"
	/// 
	/// Code Written by Tyler Moore
	/// www.FreeIndicators.Com
	/// 
	/// For modifications to this code or to have your own custom code written
	/// please visit www.MooreTechLLC.com	
    /// </summary>
    public class RoundNumberBars : BarsType
    {
        private double nextHigh = 0, nextLow = 0;
		private DateTime pTime = DateTime.MinValue;
		
		private			int		tmpCount = 0;
		
        private static bool registered = Data.BarsType.Register(new RoundNumberBars());
        
        public RoundNumberBars()
            : base(PeriodType.Custom6)
        {
			
        }

        public override object Clone() { return new RoundNumberBars(); }

        public override string DisplayName { get { return "RoundNumberBars"; } }

        public override string ToString(Period period) { return String.Format("{0} RoundNumberBars", period.Value); }

        public override void Add(Data.Bars bars, double open, double high, double low, double close, DateTime time, long volume, bool isRealtime)
        {               
            double tickSize = bars.Instrument.MasterInstrument.TickSize;
			double brickSize = Round(bars,bars.Period.Value * tickSize);  
			
			if (bars.Count < tmpCount && bars.Count > 0) // reset cache when bars are trimmed
			{
				nextHigh	= bars.GetClose(bars.Count - 1) + brickSize;
				nextLow	= bars.GetClose(bars.Count - 1) - brickSize;
			}

            // starting new bar at session boundary makes sure you have the same bars
            // reguadless of the first date that is loaded in the chart, feel free to remove			
            if ((bars.Count == 0))// || bars.IsNewSession(time))
            {                
                AddBar(bars, close, close, close, close, time, volume, isRealtime);
                nextHigh = FindNextHigh(bars,close, brickSize, tickSize);
                nextLow = FindNextLow(bars,close, brickSize, tickSize);				
            }
            else
            {
                Data.Bar bar = (Bar)bars.Get(bars.Count - 1);

                if (close>=nextHigh)
                {      					
                    UpdateBar(bars, bar.Open, nextHigh, bar.Low, nextHigh, time, volume, isRealtime);
					
					bar = (Bar)bars.Get(bars.Count - 1);
					while (close >= nextHigh)
                    {                        
                        double o = nextHigh;
                        double h = Math.Min(close, o + brickSize);
                        double l = o;
                        double c = h;                       
                        AddBar(bars, o, h, l, c, time, volume, isRealtime);
                        nextHigh += brickSize;
                        nextLow = nextHigh - 2 * brickSize;
												
						bar = (Bar)bars.Get(bars.Count - 1);												
                    }
                }
                else if (close <= nextLow)
                {
                    UpdateBar(bars, bar.Open, bar.High, nextLow, nextLow, time, volume, isRealtime);
					
					bar = (Bar)bars.Get(bars.Count - 1);
					while (close <= nextLow)
                    {
                        double o = nextLow;
                        double h = o;
                        double l = Math.Max(o-brickSize,close);
                        double c = l;
                        AddBar(bars, o, h, l, c, time, volume, isRealtime);                        
                        nextLow -= brickSize;
                        nextHigh = nextLow+2*brickSize;

						bar = (Bar)bars.Get(bars.Count - 1);						
                    }
                }
                else 
                    UpdateBar(bars, bar.Open, (close > bar.High ? close : bar.High), (close < bar.Low ? close : bar.Low), close, time, volume, isRealtime);                
            }

            bars.LastPrice = close;
			tmpCount		= bars.Count;
        }

        private double FindNextHigh(Data.Bars bars,double price, double brickSize, double tickSize)
        {
            price += tickSize;
            while (Round(bars,price % brickSize) != 0)
			{
				price += tickSize;
				price = Round(bars,price);
			}
            return price;
        }

        private double FindNextLow(Data.Bars bars,double price, double brickSize, double tickSize)
        {
            price -= tickSize;
            while (Round(bars,price % brickSize) != 0)
            {
				price -= tickSize;
				price = Round(bars,price);
			}
            return price;
        }

        private double Round(Data.Bars bars, double price)
        {
            return bars.Instrument.MasterInstrument.Round2TickSize(price);
        }

        public override PropertyDescriptorCollection GetProperties(PropertyDescriptor propertyDescriptor, Period period, Attribute[] attributes)
        {
            PropertyDescriptorCollection properties = base.GetProperties(propertyDescriptor, period, attributes);

            // here is how you remove properties not needed for that particular bars type
            properties.Remove(properties.Find("BasePeriodType", true));
            properties.Remove(properties.Find("BasePeriodValue", true));
            properties.Remove(properties.Find("PointAndFigurePriceType", true));
            properties.Remove(properties.Find("ReversalType", true));
            properties.Remove(properties.Find("Value2", true));

            // here is how you change the display name of the property on the properties grid
            Gui.Design.DisplayNameAttribute.SetDisplayName(properties, "Value", "\rBrick size");

            return properties;
        }

        public override void ApplyDefaults(Gui.Chart.BarsData barsData)
        {
            barsData.DaysBack = 3;
            barsData.Period.Value = 4;
        }

        public override int GetInitialLookBackDays(Period period, int barsBack)
        {
            return 2;  
        }

        public override int DefaultValue
        {
            get { return 5; }
        }

        public override bool IsIntraday
        {
            get { return true; }
        }

        public override PeriodType BuiltFrom
        {
            get { return PeriodType.Tick; }
        }

        public override string ChartDataBoxDate(DateTime time)
        {
            return time.ToString(Cbi.Globals.CurrentCulture.DateTimeFormat.ShortDatePattern);
        }

        public override string ChartLabel(NinjaTrader.Gui.Chart.ChartControl chartControl, DateTime time)
        {
            return time.ToString(chartControl.LabelFormatTick, Cbi.Globals.CurrentCulture);
        }

        public override double GetPercentComplete(Data.Bars bars, DateTime now)
        {           
            return 0;
        }
    }
}
